@@ -41,6 +41,7 @@
 #include <QTextBrowser>
 #include <QTextCursor>
 #include <QTextStream>
+#include <QtGlobal>
 #include <QTimer>
 #include <QVBoxLayout>
 #include "qtgui/ioconfig.h"
@@ -134,11 +135,12 @@ MainWindow::MainWindow(const QString cfgfile, bool edit_conf, QWidget *parent) :
     uiDockFft->toggleViewAction()->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_F));
     uiDockAudio->toggleViewAction()->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_A));
     uiDockBookmarks->toggleViewAction()->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_B));
+    ui->mainToolBar->toggleViewAction()->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_T));
 
-    setCorner( Qt::TopLeftCorner, Qt::LeftDockWidgetArea );
-    setCorner( Qt::TopRightCorner, Qt::RightDockWidgetArea );
-    setCorner( Qt::BottomLeftCorner, Qt::BottomDockWidgetArea );
-    setCorner( Qt::BottomRightCorner, Qt::RightDockWidgetArea );
+    setCorner(Qt::TopLeftCorner, Qt::LeftDockWidgetArea);
+    setCorner(Qt::TopRightCorner, Qt::RightDockWidgetArea);
+    setCorner(Qt::BottomLeftCorner, Qt::BottomDockWidgetArea);
+    setCorner(Qt::BottomRightCorner, Qt::RightDockWidgetArea);
 
     /* Add dock widgets to main window. This should be done even for
        dock widgets that are going to be hidden, otherwise they will
@@ -160,16 +162,9 @@ MainWindow::MainWindow(const QString cfgfile, bool edit_conf, QWidget *parent) :
     addDockWidget(Qt::BottomDockWidgetArea, uiDockBookmarks);
 
     /* hide docks that we don't want to show initially */
-    /** FIXME: Hide them initially but store layout in config **/
-    //    uiDockInputCtl->hide();
-    //    uiDockFft->hide();
-
     uiDockBookmarks->hide();
     uiDockRDS->hide();
 
-    /* misc configurations */
-    //uiDockAudio->setFftRange(0, 8000); // FM
-
     /* Add dock widget actions to View menu. By doing it this way all signal/slot
        connections will be established automagially.
     */
@@ -218,7 +213,9 @@ MainWindow::MainWindow(const QString cfgfile, bool edit_conf, QWidget *parent) :
     connect(uiDockAudio, SIGNAL(audioStreamingStarted(QString,int)), this, SLOT(startAudioStream(QString,int)));
     connect(uiDockAudio, SIGNAL(audioStreamingStopped()), this, SLOT(stopAudioStreaming()));
     connect(uiDockAudio, SIGNAL(audioRecStarted(QString)), this, SLOT(startAudioRec(QString)));
+    connect(uiDockAudio, SIGNAL(audioRecStarted(QString)), remote, SLOT(startAudioRecorder(QString)));
     connect(uiDockAudio, SIGNAL(audioRecStopped()), this, SLOT(stopAudioRec()));
+    connect(uiDockAudio, SIGNAL(audioRecStopped()), remote, SLOT(stopAudioRecorder()));
     connect(uiDockAudio, SIGNAL(audioPlayStarted(QString)), this, SLOT(startAudioPlayback(QString)));
     connect(uiDockAudio, SIGNAL(audioPlayStopped()), this, SLOT(stopAudioPlayback()));
     connect(uiDockAudio, SIGNAL(fftRateChanged(int)), this, SLOT(setAudioFftRate(int)));
@@ -232,8 +229,12 @@ MainWindow::MainWindow(const QString cfgfile, bool edit_conf, QWidget *parent) :
     connect(uiDockFft, SIGNAL(gotoFftCenter()), ui->plotter, SLOT(moveToCenterFreq()));
     connect(uiDockFft, SIGNAL(gotoDemodFreq()), ui->plotter, SLOT(moveToDemodFreq()));
 
-    connect(uiDockFft, SIGNAL(fftRangeChanged(float,float)), ui->plotter, SLOT(setFftRange(float,float)));
-    connect(ui->plotter, SIGNAL(fftRangeChanged(float,float)), uiDockFft, SLOT(setFftRange(float,float)));
+    connect(uiDockFft, SIGNAL(pandapterRangeChanged(float,float)),
+            ui->plotter, SLOT(setPandapterRange(float,float)));
+    connect(uiDockFft, SIGNAL(waterfallRangeChanged(float,float)),
+            ui->plotter, SLOT(setWaterfallRange(float,float)));
+    connect(ui->plotter, SIGNAL(pandapterRangeChanged(float,float)),
+            uiDockFft, SLOT(setPandapterRange(float,float)));
 
     connect(uiDockFft, SIGNAL(fftColorChanged(QColor)), this, SLOT(setFftColor(QColor)));
     connect(uiDockFft, SIGNAL(fftFillToggled(bool)), this, SLOT(setFftFill(bool)));
@@ -242,9 +243,7 @@ MainWindow::MainWindow(const QString cfgfile, bool edit_conf, QWidget *parent) :
     connect(uiDockRDS, SIGNAL(rdsDecoderToggled(bool)), this, SLOT(setRdsDecoder(bool)));
 
     // Bookmarks
-    connect(uiDockBookmarks, SIGNAL(newFrequency(qint64)), this, SLOT(setNewFrequency(qint64)));
-    connect(uiDockBookmarks, SIGNAL(newDemodulation(QString)), this, SLOT(selectDemod(QString)));
-    connect(uiDockBookmarks, SIGNAL(newFilterBandwidth(int, int)), this, SLOT(on_plotter_newFilterFreq(int, int)));
+    connect(uiDockBookmarks, SIGNAL(newBookmarkActivated(qint64, QString, int)), this, SLOT(onBookmarkActivated(qint64, QString, int)));
     connect(uiDockBookmarks->actionAddBookmark, SIGNAL(triggered()), this, SLOT(on_actionAddBookmark_triggered()));
 
 
@@ -264,10 +263,8 @@ MainWindow::MainWindow(const QString cfgfile, bool edit_conf, QWidget *parent) :
     connect(remote, SIGNAL(newSquelchLevel(double)), this, SLOT(setSqlLevel(double)));
     connect(remote, SIGNAL(newSquelchLevel(double)), uiDockRxOpt, SLOT(setSquelchLevel(double)));
     connect(uiDockRxOpt, SIGNAL(sqlLevelChanged(double)), remote, SLOT(setSquelchLevel(double)));
-
-    // satellite events
-    connect(remote, SIGNAL(satAosEvent()), uiDockAudio, SLOT(startAudioRecorder()));
-    connect(remote, SIGNAL(satLosEvent()), uiDockAudio, SLOT(stopAudioRecorder()));
+    connect(remote, SIGNAL(startAudioRecorderEvent()), uiDockAudio, SLOT(startAudioRecorder()));
+    connect(remote, SIGNAL(stopAudioRecorderEvent()), uiDockAudio, SLOT(stopAudioRecorder()));
 
     rds_timer = new QTimer(this);
     connect(rds_timer, SIGNAL(timeout()), this, SLOT(rdsTimeout()));
@@ -279,8 +276,12 @@ MainWindow::MainWindow(const QString cfgfile, bool edit_conf, QWidget *parent) :
     ui->plotter->setTooltipsEnabled(true);
 #endif
 
+    // Create list of input devices. This must be done before the configuration is
+    // restored because device probing might change the device configuration
+    CIoConfig::getDeviceList(devList);
+
     // restore last session
-    if (!loadConfig(cfgfile, true))
+    if (!loadConfig(cfgfile, true, true))
     {
 
       // first time config
@@ -352,6 +353,7 @@ MainWindow::~MainWindow()
     delete ui;
     delete uiDockRxOpt;
     delete uiDockAudio;
+    delete uiDockBookmarks;
     delete uiDockFft;
     delete uiDockInputCtl;
     delete uiDockRDS;
@@ -368,17 +370,18 @@ MainWindow::~MainWindow()
  * @param cfgfile
  * @returns True if config is OK, False if not (e.g. no input device specified).
  *
- * If cfgfile is an absolute path it will be used as is, otherwise it is assumed to be the
- * name of a file under m_cfg_dir.
+ * If cfgfile is an absolute path it will be used as is, otherwise it is assumed
+ * to be the name of a file under m_cfg_dir.
  *
  * If cfgfile does not exist it will be created.
  *
- * If no input device is specified, we return false to signal that the I/O configuration
- * dialog should be run.
+ * If no input device is specified, we return false to signal that the I/O
+ * configuration dialog should be run.
  *
  * FIXME: Refactor.
  */
-bool MainWindow::loadConfig(const QString cfgfile, bool check_crash)
+bool MainWindow::loadConfig(const QString cfgfile, bool check_crash,
+                            bool restore_mainwindow)
 {
     double      actual_rate;
     qint64      int64_val;
@@ -396,7 +399,8 @@ bool MainWindow::loadConfig(const QString cfgfile, bool check_crash)
     if (QDir::isAbsolutePath(cfgfile))
         m_settings = new QSettings(cfgfile, QSettings::IniFormat);
     else
-        m_settings = new QSettings(QString("%1/%2").arg(m_cfg_dir).arg(cfgfile), QSettings::IniFormat);
+        m_settings = new QSettings(QString("%1/%2").arg(m_cfg_dir).arg(cfgfile),
+                                   QSettings::IniFormat);
 
     qDebug() << "Configuration file:" << m_settings->fileName();
 
@@ -438,8 +442,12 @@ bool MainWindow::loadConfig(const QString cfgfile, bool check_crash)
         ui->mainToolBar->hide();
 
     // main window settings
-    restoreGeometry(m_settings->value("gui/geometry", saveGeometry()).toByteArray());
-    restoreState(m_settings->value("gui/state", saveState()).toByteArray());
+    if (restore_mainwindow)
+    {
+        restoreGeometry(m_settings->value("gui/geometry",
+                                          saveGeometry()).toByteArray());
+        restoreState(m_settings->value("gui/state", saveState()).toByteArray());
+    }
 
     QString indev = m_settings->value("input/device", "").toString();
     if (!indev.isEmpty())
@@ -472,9 +480,7 @@ bool MainWindow::loadConfig(const QString cfgfile, bool check_crash)
             updateGainStages(false);
         }
         else
-        {
             updateGainStages(true);
-        }
     }
 
     QString outdev = m_settings->value("output/device", "").toString();
@@ -506,9 +512,7 @@ bool MainWindow::loadConfig(const QString cfgfile, bool check_crash)
         qDebug() << "Actual sample rate   :" << QString("%1").arg(actual_rate, 0, 'f', 6);
     }
     else
-    {
         actual_rate = rx->get_input_rate();
-    }
 
     if (actual_rate > 0.)
     {
@@ -529,9 +533,8 @@ bool MainWindow::loadConfig(const QString cfgfile, bool check_crash)
             }
         }
         else
-        {
             rx->set_input_decim(1);
-        }
+
         // update various widgets that need a sample rate
         uiDockRxOpt->setFilterOffsetRange((qint64)(actual_rate));
         uiDockFft->setSampleRate(actual_rate);
@@ -541,9 +544,7 @@ bool MainWindow::loadConfig(const QString cfgfile, bool check_crash)
         iq_tool->setSampleRate((qint64)actual_rate);
     }
     else
-    {
         qDebug() << "Error: Actual sample rate is" << actual_rate;
-    }
 
     int64_val = m_settings->value("input/bandwidth", 0).toInt(&conv_ok);
     if (conv_ok)
@@ -585,11 +586,11 @@ bool MainWindow::loadConfig(const QString cfgfile, bool check_crash)
  * @param cfgfile
  * @returns True if the operation was successful.
  *
- * If cfgfile is an absolute path it will be used as is, otherwise it is assumed to be the
- * name of a file under m_cfg_dir.
+ * If cfgfile is an absolute path it will be used as is, otherwise it is
+ * assumed to be the name of a file under m_cfg_dir.
  *
- * If cfgfile already exists it will be overwritten (we assume that a file selection dialog
- * has already asked for confirmation of overwrite.
+ * If cfgfile already exists it will be overwritten (we assume that a file
+ * selection dialog has already asked for confirmation of overwrite.
  *
  * Since QSettings does not support "save as" we do this by copying the current
  * settings to a new file.
@@ -622,7 +623,7 @@ bool MainWindow::saveConfig(const QString cfgfile)
         m_settings->setValue("crashed", false);
         m_settings->sync();
 
-        loadConfig(cfgfile, false);
+        loadConfig(cfgfile, false, false);
         return true;
     }
     else
@@ -747,7 +748,7 @@ void MainWindow::updateGainStages(bool read_from_device)
  */
 void MainWindow::setNewFrequency(qint64 rx_freq)
 {
-    double hw_freq = (double)(rx_freq-d_lnb_lo) - rx->get_filter_offset();
+    double hw_freq = (double)(rx_freq - d_lnb_lo) - rx->get_filter_offset();
     qint64 center_freq = rx_freq - (qint64)rx->get_filter_offset();
 
     d_hw_freq = (qint64)hw_freq;
@@ -842,9 +843,9 @@ void MainWindow::setAutoGain(bool enabled)
 void MainWindow::setFreqCorr(double ppm)
 {
     if (ppm < -200.0)
-   ppm = -200.0;
+        ppm = -200.0;
     else if (ppm > 200.0)
-   ppm = 200.0;
+        ppm = 200.0;
 
     qDebug() << __FUNCTION__ << ":" << ppm << "ppm";
     rx->set_freq_corr(ppm);
@@ -934,9 +935,8 @@ void MainWindow::selectDemod(int mode_idx)
     d_filter_shape = (receiver::filter_shape)uiDockRxOpt->currentFilterShape();
 
     if (rx->is_rds_decoder_active())
-    {
         setRdsDecoder(false);
-    }
+
     uiDockRDS->setDisabled();
 
     switch (mode_idx) {
@@ -955,10 +955,12 @@ void MainWindow::selectDemod(int mode_idx)
 
     case DockRxOpt::MODE_RAW:
         /* Raw I/Q */
-        qDebug() << "RAW I/Q mode not implemented!";
+        rx->set_demod(receiver::RX_DEMOD_NONE);
+        ui->plotter->setDemodRanges(-45000, -200, 200, 45000, true);
+        uiDockAudio->setFftRange(0,24000);
+        click_res = 100;
         break;
 
-        /* AM */
     case DockRxOpt::MODE_AM:
         rx->set_demod(receiver::RX_DEMOD_AM);
         ui->plotter->setDemodRanges(-45000, -200, 200, 45000, true);
@@ -966,7 +968,6 @@ void MainWindow::selectDemod(int mode_idx)
         click_res = 100;
         break;
 
-        /* Narrow FM */
     case DockRxOpt::MODE_NFM:
         rx->set_demod(receiver::RX_DEMOD_NFM);
         click_res = 100;
@@ -983,10 +984,10 @@ void MainWindow::selectDemod(int mode_idx)
         }
         break;
 
-        /* Broadcast FM */
     case DockRxOpt::MODE_WFM_MONO:
     case DockRxOpt::MODE_WFM_STEREO:
     case DockRxOpt::MODE_WFM_STEREO_OIRT:
+        /* Broadcast FM */
         quad_rate = rx->get_input_rate();
         if (quad_rate < 500.0e3)
             ui->plotter->setDemodRanges(-quad_rate/2.0, -10000,
@@ -1006,24 +1007,24 @@ void MainWindow::selectDemod(int mode_idx)
         uiDockRDS->setEnabled();
         break;
 
-        /* LSB */
     case DockRxOpt::MODE_LSB:
+        /* LSB */
         rx->set_demod(receiver::RX_DEMOD_SSB);
         ui->plotter->setDemodRanges(-40000, -100, -5000, 0, false);
         uiDockAudio->setFftRange(0,3000);
         click_res = 100;
         break;
 
-        /* USB */
     case DockRxOpt::MODE_USB:
+        /* USB */
         rx->set_demod(receiver::RX_DEMOD_SSB);
         ui->plotter->setDemodRanges(0, 5000, 100, 40000, false);
         uiDockAudio->setFftRange(0,3000);
         click_res = 100;
         break;
 
-        /* CW-L */
     case DockRxOpt::MODE_CWL:
+        /* CW-L */
         rx->set_demod(receiver::RX_DEMOD_SSB);
         cwofs = -uiDockRxOpt->getCwOffset();
         ui->plotter->setDemodRanges(-5000, -100, 100, 5000, true);
@@ -1031,8 +1032,8 @@ void MainWindow::selectDemod(int mode_idx)
         click_res = 10;
         break;
 
-        /* CW-U */
     case DockRxOpt::MODE_CWU:
+        /* CW-U */
         rx->set_demod(receiver::RX_DEMOD_SSB);
         cwofs = uiDockRxOpt->getCwOffset();
         ui->plotter->setDemodRanges(-5000, -100, 100, 5000, true);
@@ -1054,8 +1055,9 @@ void MainWindow::selectDemod(int mode_idx)
     ui->plotter->setFilterClickResolution(click_res);
     rx->set_filter((double)flo, (double)fhi, d_filter_shape);
     rx->set_cw_offset(cwofs);
+    rx->set_sql_level(uiDockRxOpt->currentSquelchLevel());
 
-    d_have_audio = ((mode_idx != DockRxOpt::MODE_OFF) && (mode_idx != DockRxOpt::MODE_RAW));
+    d_have_audio = (mode_idx != DockRxOpt::MODE_OFF);
 
     uiDockRxOpt->setCurrentDemod(mode_idx);
 }
@@ -1261,7 +1263,7 @@ void MainWindow::audioFftTimeout()
     float           pwr_scale;
     std::complex<float> pt;             /* a single FFT point used in calculations */
 
-    if (!d_have_audio)
+    if (!d_have_audio || !uiDockAudio->isVisible())
         return;
 
     rx->get_audio_fft_data(d_fftData, fftsize);
@@ -1408,9 +1410,10 @@ void MainWindow::startIqRecording(const QString recdir)
     // gqrx_iq_yyyymmdd_hhmmss_freq_bw_fc.raw
     qint64 freq = (qint64)(rx->get_rf_freq());
     qint64 sr = (qint64)(rx->get_input_rate());
+    qint32 dec = (quint32)(rx->get_input_decim());
     QString lastRec = QDateTime::currentDateTimeUtc().
             toString("%1/gqrx_yyyyMMdd_hhmmss_%2_%3_fc.'raw'")
-            .arg(recdir).arg(freq).arg(sr);
+            .arg(recdir).arg(freq).arg(sr/dec);
 
     // start recorder; fails if recording already in progress
     if (rx->start_iq_recording(lastRec.toStdString()))
@@ -1639,11 +1642,11 @@ void MainWindow::setPeakDetection(bool enabled)
  * started. The jerkyness disappears when trhe receiver is reconfigured
  * by selecting a new demodulator.
  */
-void MainWindow::forceRxReconf()
+/*void MainWindow::forceRxReconf()
 {
     qDebug() << "Force RX reconf (jerky dongle workarond)...";
     selectDemod(uiDockRxOpt->currentDemod());
-}
+}*/
 
 /**
  * @brief Start/Stop DSP processing.
@@ -1654,6 +1657,8 @@ void MainWindow::forceRxReconf()
  */
 void MainWindow::on_actionDSP_triggered(bool checked)
 {
+    remote->setReceiverStatus(checked);
+
     if (checked)
     {
         /* start receiver */
@@ -1713,7 +1718,7 @@ int MainWindow::on_actionIoConfig_triggered()
 {
     qDebug() << "Configure I/O devices.";
 
-    CIoConfig *ioconf = new CIoConfig(m_settings);
+    CIoConfig *ioconf = new CIoConfig(m_settings, devList);
     int confres = ioconf->exec();
 
     if (confres == QDialog::Accepted)
@@ -1722,8 +1727,10 @@ int MainWindow::on_actionIoConfig_triggered()
             // suspend DSP while we reload settings
             on_actionDSP_triggered(false);
 
+        // Refresh LNB LO in dock widget, otherwise changes will be lost
+        uiDockInputCtl->readLnbLoFromSettings(m_settings);
         storeSession();
-        loadConfig(m_settings->fileName(), false);
+        loadConfig(m_settings->fileName(), false, false);
 
         if (ui->actionDSP->isChecked())
             // restsart DSP
@@ -1741,11 +1748,11 @@ int MainWindow::firstTimeConfig()
 {
     qDebug() << __func__;
 
-    CIoConfig *ioconf = new CIoConfig(m_settings);
+    CIoConfig *ioconf = new CIoConfig(m_settings, devList);
     int confres = ioconf->exec();
 
     if (confres == QDialog::Accepted)
-        loadConfig(m_settings->fileName(), false);
+        loadConfig(m_settings->fileName(), false, false);
 
     delete ioconf;
 
@@ -1769,7 +1776,7 @@ void MainWindow::on_actionLoadSettings_triggered()
     if (!cfgfile.endsWith(".conf", Qt::CaseSensitive))
         cfgfile.append(".conf");
 
-    loadConfig(cfgfile, cfgfile != m_settings->fileName());
+    loadConfig(cfgfile, cfgfile != m_settings->fileName(), cfgfile != m_settings->fileName());
 
     // store last dir
     QFileInfo fi(cfgfile);
@@ -1804,7 +1811,7 @@ void MainWindow::on_actionSaveSettings_triggered()
 
 void MainWindow::on_actionSaveWaterfall_triggered()
 {
-    QDateTime   dt(QDateTime::currentDateTime());
+    QDateTime   dt(QDateTime::currentDateTimeUtc());
     QString     wffile;
     QString     save_path;
 
@@ -1848,9 +1855,8 @@ void MainWindow::on_plotter_newDemodFreq(qint64 freq, qint64 delta)
     uiDockRxOpt->setFilterOffset(delta);
     ui->freqCtrl->setFrequency(freq);
 
-    if (rx->is_rds_decoder_active()) {
+    if (rx->is_rds_decoder_active())
         rx->reset_rds_parser();
-    }
 }
 
 /* CPlotter::NewfilterFreq() is emitted */
@@ -1861,10 +1867,12 @@ void MainWindow::on_plotter_newFilterFreq(int low, int high)
     /* parameter correctness will be checked in receiver class */
     retcode = rx->set_filter((double) low, (double) high, d_filter_shape);
 
+    /* Update filter range of plotter, in case this slot is triggered by
+     * switching to a bookmark */
+    ui->plotter->setHiLowCutFrequencies(low, high);
+
     if (retcode == receiver::STATUS_OK)
-    {
         uiDockRxOpt->setFilterParam(low, high);
-    }
 }
 
 void MainWindow::on_plotter_newCenterFreq(qint64 f)
@@ -1946,12 +1954,10 @@ void MainWindow::on_actionAFSK1200_triggered()
             dec_timer->start(100);
         }
         else
-        {
             QMessageBox::warning(this, tr("Gqrx error"),
                                  tr("Error starting sample sniffer.\n"
                                     "Close all data decoders and try again."),
                                  QMessageBox::Ok, QMessageBox::Ok);
-        }
     }
 }
 
@@ -1984,14 +1990,9 @@ void MainWindow::decoderTimeout()
     float buffer[DATA_BUFFER_SIZE];
     unsigned int num;
 
-    //qDebug() << "Process decoder";
-
     rx->get_sniffer_data(&buffer[0], num);
     if (dec_afsk1200)
-    {
         dec_afsk1200->process_samples(&buffer[0], num);
-    }
-    /* else stop timeout and sniffer? */
 }
 
 void MainWindow::setRdsDecoder(bool checked)
@@ -2013,18 +2014,46 @@ void MainWindow::setRdsDecoder(bool checked)
     }
 }
 
+void MainWindow::onBookmarkActivated(qint64 freq, QString demod, int bandwidth)
+{
+    setNewFrequency(freq);
+    selectDemod(demod);
+
+    /* Check if filter is symmetric or not by checking the presets */
+    int mode = uiDockRxOpt->currentDemod();
+    int preset = uiDockRxOpt->currentFilterShape();
+
+    int lo, hi;
+    uiDockRxOpt->getFilterPreset(mode, preset, &lo, &hi);
+
+    if(lo + hi == 0)
+    {
+        lo = -bandwidth / 2;
+        hi =  bandwidth / 2;
+    }
+    else if(lo >= 0 && hi >= 0)
+    {
+        hi = lo + bandwidth;
+    }
+    else if(lo <= 0 && hi <= 0)
+    {
+        lo = hi - bandwidth;
+    }
+
+    on_plotter_newFilterFreq(lo, hi);
+    ui->plotter->setHiLowCutFrequencies(lo, hi);
+}
+
 /** Launch Gqrx google group website. */
 void MainWindow::on_actionUserGroup_triggered()
 {
     bool res = QDesktopServices::openUrl(QUrl("https://groups.google.com/forum/#!forum/gqrx",
                                               QUrl::TolerantMode));
     if (!res)
-    {
         QMessageBox::warning(this, tr("Error"),
                              tr("Failed to open website:\n"
                                 "https://groups.google.com/forum/#!forum/gqrx"),
                              QMessageBox::Close);
-    }
 }
 
 /**
@@ -2091,7 +2120,7 @@ void MainWindow::showSimpleTextFile(const QString &resource_path,
 }
 
 /**
- * @brief Action: About Qthid
+ * @brief Action: About gqrx.
  *
  * This slot is called when the user activates the
  * Help|About menu item (or Gqrx|About on Mac)
@@ -2100,27 +2129,19 @@ void MainWindow::on_actionAbout_triggered()
 {
     QMessageBox::about(this, tr("About Gqrx"),
         tr("<p>This is Gqrx %1</p>"
-          "<p>Copyright (C) 2011-2016 Alexandru Csete & contributors.</p>"
-          "<p>Gqrx is a software defined radio receiver powered by "
-          "<a href='http://www.gnuradio.org/'>GNU Radio</a> and the Qt toolkit. "
-          "<p>Gqrx uses the <a href='http://sdr.osmocom.org/trac/wiki/GrOsmoSDR'>GrOsmoSDR</a> "
-          "input source block and and works with any input device supported by it including:"
-          "<ul>"
-          "<li><a href='http://funcubedongle.com/'>Funcube Dongle Pro and Pro+</a></li>"
-          "<li><a href='http://sdr.osmocom.org/trac/wiki/rtl-sdr'>RTL2832U-based DVB-T tuners (rtlsdr and rtlsdr-tcp)</a></li>"
-          "<li><a href='http://www.ettus.com/'>Ettus Research USRP devices</a></li>"
-          "<li><a href='http://sdr.osmocom.org/trac/'>OsmoSDR</a></li>"
-          "<li><a href='https://greatscottgadgets.com/hackrf/'>HackRF One & Jawbreaker</a></li>"
-          "<li><a href='http://nuand.com/bladeRF'>Nuand bladeRF</a></li>"
-          "<li><a href='http://airspy.com'>Airspy</a></li>"
-          "<li><a href='http://rfspace.com'>RFspace receivers</a></li>"
-          "</ul></p>"
-          "<p>You can download the latest version from the "
-          "<a href='http://gqrx.dk/'>Gqrx website</a>."
-          "</p>"
-          "<p>"
-          "Gqrx is licensed under the <a href='http://www.gnu.org/licenses/gpl.html'>GNU General Public License</a>."
-          "</p>").arg(VERSION));
+           "<p>Copyright (C) 2011-2016 Alexandru Csete & contributors.</p>"
+           "<p>Gqrx is a software defined radio (SDR) receiver powered by "
+           "<a href='http://www.gnuradio.org/'>GNU Radio</a> and the Qt toolkit. "
+           "<p>Gqrx uses the <a href='http://sdr.osmocom.org/trac/wiki/GrOsmoSDR'>GrOsmoSDR</a> "
+           "input source block and and works with any input device supported by it, including "
+           "Funcube Dongles, RTL-SDR, Airspy, HackRF, RFSpace, BladeRF and USRP receivers."
+           "</p>"
+           "<p>You can download the latest version from the "
+           "<a href='http://gqrx.dk/'>Gqrx website</a>."
+           "</p>"
+           "<p>"
+           "Gqrx is licensed under the <a href='http://www.gnu.org/licenses/gpl.html'>GNU General Public License</a>."
+           "</p>").arg(VERSION));
 }
 
 /**
@@ -2199,17 +2220,15 @@ void MainWindow::on_actionAddBookmark_triggered()
         QStringList listTags = tags.split(",",QString::SkipEmptyParts);
         info.tags.clear();
         if (listTags.size() == 0)
-        {
             info.tags.append(&Bookmarks::Get().findOrAddTag(""));
-        }
-        for(i = 0; i < listTags.size(); ++i)
-        {
+
+
+        for (i = 0; i < listTags.size(); ++i)
             info.tags.append(&Bookmarks::Get().findOrAddTag(listTags[i]));
-        }
+
         Bookmarks::Get().add(info);
         uiDockBookmarks->updateTags();
         uiDockBookmarks->updateBookmarks();
         ui->plotter->updateOverlay();
     }
-
 }
