@@ -22,7 +22,9 @@
  */
 #include <cmath>
 #include <iostream>
+#ifndef _MSC_VER
 #include <unistd.h>
+#endif
 
 #include <iostream>
 
@@ -34,13 +36,16 @@
 
 #include "applications/gqrx/receiver.h"
 #include "dsp/correct_iq_cc.h"
-#include "dsp/hbf_decim.h"
+//#include "dsp/hbf_decim.h"
+#include "dsp/filter/fir_decim.h"
 #include "dsp/rx_fft.h"
 #include "receivers/nbrx.h"
 #include "receivers/wfmrx.h"
 
 #ifdef WITH_PULSEAUDIO
 #include "pulseaudio/pa_sink.h"
+#elif WITH_PORTAUDIO
+#include "portaudio/portaudio_sink.h"
 #else
 #include <gnuradio/audio/sink.h>
 #endif
@@ -61,6 +66,7 @@ receiver::receiver(const std::string input_device,
       d_decim(decimation),
       d_rf_freq(144800000.0),
       d_filter_offset(0.0),
+      d_cw_offset(0.0),
       d_recording_iq(false),
       d_recording_wav(false),
       d_sniffer_active(false),
@@ -74,8 +80,7 @@ receiver::receiver(const std::string input_device,
 
     if (input_device.empty())
     {
-        // FIXME: other OS
-        src = osmosdr::source::make("file=/dev/random,freq=428e6,rate=96000,repeat=true,throttle=true");
+        src = osmosdr::source::make("file="+get_random_file()+",freq=428e6,rate=96000,repeat=true,throttle=true");
     }
     else
     {
@@ -88,7 +93,7 @@ receiver::receiver(const std::string input_device,
     {
         try
         {
-            input_decim = make_hbf_decim(d_decim);
+            input_decim = make_fir_decim_cc(d_decim);
         }
         catch (std::range_error &e)
         {
@@ -107,7 +112,7 @@ receiver::receiver(const std::string input_device,
 
 
     // create I/Q sink and close it
-    iq_sink = gr::blocks::file_sink::make(sizeof(gr_complex), "/dev/null", false);
+    iq_sink = gr::blocks::file_sink::make(sizeof(gr_complex), get_null_file().c_str(), true);
     iq_sink->set_unbuffered(true);
     iq_sink->close();
 
@@ -124,7 +129,7 @@ receiver::receiver(const std::string input_device,
     audio_gain0 = gr::blocks::multiply_const_ff::make(0.1);
     audio_gain1 = gr::blocks::multiply_const_ff::make(0.1);
 
-    wav_sink = gr::blocks::wavfile_sink::make("/dev/null", 2,
+    wav_sink = gr::blocks::wavfile_sink::make(get_null_file().c_str(), 2,
                                               (unsigned int) d_audio_rate,
                                               16);
 
@@ -132,6 +137,8 @@ receiver::receiver(const std::string input_device,
 
 #ifdef WITH_PULSEAUDIO
     audio_snk = make_pa_sink(audio_device, d_audio_rate, "GQRX", "Audio output");
+#elif WITH_PORTAUDIO
+    audio_snk = make_portaudio_sink(audio_device, d_audio_rate, "GQRX", "Audio output");
 #else
     audio_snk = gr::audio::sink::make(d_audio_rate, audio_device, true);
 #endif
@@ -224,6 +231,8 @@ void receiver::set_input_device(const std::string device)
 
     src.reset();
     src = osmosdr::source::make(device);
+    if(src->get_sample_rate() != 0)
+        set_input_rate(src->get_sample_rate());
 
     if (d_decim >= 2)
     {
@@ -269,6 +278,8 @@ void receiver::set_output_device(const std::string device)
 
 #ifdef WITH_PULSEAUDIO
     audio_snk = make_pa_sink(device, d_audio_rate, "GQRX", "Audio output");
+#elif WITH_PORTAUDIO
+    audio_snk = make_portaudio_sink(device, d_audio_rate, "GQRX", "Audio output");
 #else
     audio_snk = gr::audio::sink::make(d_audio_rate, device, true);
 #endif
@@ -372,7 +383,7 @@ unsigned int receiver::set_input_decim(unsigned int decim)
     {
         try
         {
-            input_decim = make_hbf_decim(d_decim);
+            input_decim = make_fir_decim_cc(d_decim);
         }
         catch (std::range_error &e)
         {
@@ -404,6 +415,11 @@ unsigned int receiver::set_input_decim(unsigned int decim)
         tb->connect(src, 0, iq_swap, 0);
     }
 
+#ifdef CUSTOM_AIRSPY_KERNELS
+    if (input_devstr.find("airspy") != std::string::npos)
+        src->set_bandwidth(d_quad_rate);
+#endif
+
     if (d_running)
         tb->start();
 
@@ -625,13 +641,13 @@ receiver::status receiver::set_auto_gain(bool automatic)
 receiver::status receiver::set_filter_offset(double offset_hz)
 {
     d_filter_offset = offset_hz;
-    lo->set_frequency(-d_filter_offset);
+    lo->set_frequency(-d_filter_offset + d_cw_offset);
 
     return STATUS_OK;
 }
 
 /**
- * @brief Get filterm offset.
+ * @brief Get filter offset.
  * @return The current filter offset.
  * @sa set_filter_offset()
  */
@@ -640,6 +656,21 @@ double receiver::get_filter_offset(void) const
     return d_filter_offset;
 }
 
+/* CW offset can serve as a "BFO" if the GUI needs it */
+receiver::status receiver::set_cw_offset(double offset_hz)
+{
+    d_cw_offset = offset_hz;
+    lo->set_frequency(-d_filter_offset + d_cw_offset);
+    rx->set_cw_offset(d_cw_offset);
+
+    return STATUS_OK;
+}
+
+double receiver::get_cw_offset(void) const
+{
+    return d_cw_offset;
+}
+
 receiver::status receiver::set_filter(double low, double high, filter_shape shape)
 {
     double trans_width;
@@ -650,11 +681,11 @@ receiver::status receiver::set_filter(double low, double high, filter_shape shap
     switch (shape) {
 
     case FILTER_SHAPE_SOFT:
-        trans_width = std::abs(high - low) * 0.4;
+        trans_width = std::abs(high - low) * 0.5;
         break;
 
     case FILTER_SHAPE_SHARP:
-        trans_width = std::abs(high - low) * 0.05;
+        trans_width = std::abs(high - low) * 0.1;
         break;
 
     case FILTER_SHAPE_NORMAL:
