@@ -36,6 +36,9 @@ RemoteControl::RemoteControl(QObject *parent) :
     bw_half = 740e3;
     rc_mode = 0;
     signal_level = -200.0;
+    squelch_level = -150.0;
+    audio_recorder_status = false;
+    receiver_running = false;
 
     rc_port = 7356;
     rc_allowed_hosts.append("::ffff:127.0.0.1");
@@ -161,19 +164,25 @@ void RemoteControl::startRead()
 {
     char    buffer[1024] = {0};
     int     bytes_read;
-    qint64  freq;
 
+    bool ok = true;
 
     bytes_read = rc_socket->readLine(buffer, 1024);
     if (bytes_read < 2)  // command + '\n'
         return;
 
-    if (buffer[0] == 'F')
+    QStringList cmdlist = QString(buffer).trimmed().split(" ", QString::SkipEmptyParts);
+
+    if (cmdlist.size() == 0)
+        return;
+
+    // Set new frequency
+    if (cmdlist[0] == "F")
     {
-        // set frequency
-        if (sscanf(buffer,"F %lld\n", &freq) == 1)
+        double freq = cmdlist.value(1, "ERR").toDouble(&ok);
+        if (ok)
         {
-            setNewRemoteFreq(freq);
+            setNewRemoteFreq((qint64)freq);
             rc_socket->write("RPRT 0\n");
         }
         else
@@ -181,28 +190,60 @@ void RemoteControl::startRead()
             rc_socket->write("RPRT 1\n");
         }
     }
-    else if (buffer[0] == 'f')
+    // Get frequency
+    else if (cmdlist[0] == "f")
     {
-        // get frequency
         rc_socket->write(QString("%1\n").arg(rc_freq).toLatin1());
     }
-    else if (buffer[0] == 'c')
+    else if (cmdlist[0] == "c")
     {
         // FIXME: for now we assume 'close' command
         rc_socket->close();
     }
-
-    // Get level
-    // FIXME: For now only signal strength is returned
-    else if (buffer[0] == 'l')
+    // Set level
+    else if (cmdlist[0] == "L")
     {
-        rc_socket->write(QString("%1\n").arg(signal_level, 0, 'f', 1).toLatin1());
+        QString lvl = cmdlist.value(1, "");
+        if (lvl == "?")
+        {
+            rc_socket->write("SQL\n");
+        }
+        else if (lvl.compare("SQL", Qt::CaseInsensitive) == 0)
+        {
+            double squelch = cmdlist.value(2, "ERR").toDouble(&ok);
+            if (ok)
+            {
+                rc_socket->write("RPRT 0\n");
+                squelch_level = std::max<double>(-150, std::min<double>(0, squelch));
+                emit newSquelchLevel(squelch_level);
+            }
+            else
+            {
+                rc_socket->write("RPRT 1\n");
+            }
+        }
+        else
+        {
+            rc_socket->write("RPRT 1\n");
+        }
     }
-
-    // Mode and filter
-    else if (buffer[0] == 'M')
+    // Get level
+    else if (cmdlist[0] == "l")
     {
-        int mode = modeStrToInt(buffer);
+        QString lvl = cmdlist.value(1, "");
+        if (lvl == "?")
+            rc_socket->write("SQL STRENGTH\n");
+        else if (lvl.compare("STRENGTH", Qt::CaseInsensitive) == 0 || lvl.isEmpty())
+            rc_socket->write(QString("%1\n").arg(signal_level, 0, 'f', 1).toLatin1());
+        else if (lvl.compare("SQL", Qt::CaseInsensitive) == 0)
+            rc_socket->write(QString("%1\n").arg(squelch_level, 0, 'f', 1).toLatin1());
+        else
+            rc_socket->write("RPRT 1\n");
+    }
+    // Mode and filter
+    else if (cmdlist[0] == "M")
+    {
+        int mode = modeStrToInt(cmdlist.value(1, ""));
         if (mode == -1)
         {
             // invalid string
@@ -212,39 +253,122 @@ void RemoteControl::startRead()
         {
             rc_socket->write("RPRT 0\n");
             rc_mode = mode;
+
+            if (rc_mode == 0)
+                audio_recorder_status = false;
+
             emit newMode(rc_mode);
         }
     }
-    else if (buffer[0] == 'm')
+    else if (cmdlist[0] == "m")
     {
         rc_socket->write(QString("%1\n").arg(intToModeStr(rc_mode)).toLatin1());
     }
-
-
-    // Gpredict / Gqrx specific commands:
-    //   AOS  - satellite AOS event
-    //   LOS  - satellite LOS event
-    else if (bytes_read >= 4 && buffer[1] == 'O' && buffer[2] == 'S')
+    else if (cmdlist[0] == "U")
     {
-        if (buffer[0] == 'A')
+        QString func = cmdlist.value(1, "");
+        bool ok;
+        int status = cmdlist.value(2, "").toInt(&ok);
+
+        if (func == "?")
         {
-            emit satAosEvent();
-            rc_socket->write("RPRT 0\n");
+            rc_socket->write("RECORD\n");
         }
-        else if (buffer[0] == 'L')
+        else if (func == "" || !ok)
         {
-            emit satLosEvent();
-            rc_socket->write("RPRT 0\n");
+            rc_socket->write("RPRT 1\n");
+        }
+        else if (func.compare("RECORD", Qt::CaseInsensitive) == 0)
+        {
+            if (rc_mode == 0 || !receiver_running)
+            {
+                rc_socket->write("RPRT 1\n");
+            }
+            else
+            {
+                rc_socket->write("RPRT 0\n");
+                audio_recorder_status = status;
+                if (status)
+                    emit startAudioRecorderEvent();
+                else
+                    emit stopAudioRecorderEvent();
+            }
         }
         else
         {
             rc_socket->write("RPRT 1\n");
         }
     }
+    else if (cmdlist[0] == "u")
+    {
+        QString func = cmdlist.value(1, "");
+
+        if (func == "?")
+            rc_socket->write("RECORD\n");
+        else if (func.compare("RECORD", Qt::CaseInsensitive) == 0)
+            rc_socket->write(QString("%1\n").arg(audio_recorder_status).toLatin1());
+        else
+            rc_socket->write("RPRT 1\n");
+    }
+
+
+    // Gpredict / Gqrx specific commands:
+    //   AOS  - satellite AOS event
+    //   LOS  - satellite LOS event
+    else if (cmdlist[0] == "AOS")
+    {
+        if (rc_mode > 0 && receiver_running)
+        {
+            emit startAudioRecorderEvent();
+            audio_recorder_status = true;
+        }
+        rc_socket->write("RPRT 0\n");
+
+    }
+    else if (cmdlist[0] == "LOS")
+    {
+        emit stopAudioRecorderEvent();
+        audio_recorder_status = false;
+        rc_socket->write("RPRT 0\n");
+
+    }
+
+    /* dump_state used by some clients, e.g. xdx
+     * For now just some quick hack that works taken from
+     * https://github.com/hexameron/rtl-sdrangelove/blob/master/plugins/channel/tcpsrc/rigctl.cpp
+     *
+     * More info in tests/rigctl_parse.c
+     */
+    else if (cmdlist[0] == "\\dump_state")
+    {
+        rc_socket->write("0\n"
+                         "2\n"
+                         "1\n"
+                         "150000.000000 30000000.000000  0x900af -1 -1 0x10000003 0x3\n"
+                         "0 0 0 0 0 0 0\n"
+                         "150000.000000 30000000.000000  0x900af -1 -1 0x10000003 0x3\n"
+                         "0 0 0 0 0 0 0\n"
+                         "0 0\n"
+                         "0 0\n"
+                         "0\n"
+                         "0\n"
+                         "0\n"
+                         "0\n"
+                         "\n"
+                         "\n"
+                         "0x0\n"
+                         "0x0\n"
+                         "0x0\n"
+                         "0x0\n"
+                         "0x0\n"
+                         "0\n");
+    }
 
     else
     {
-        // respond with an error
+        // print unknown command and respond with an error
+        qWarning() << "Unknown remote command:"
+                << cmdlist;
         rc_socket->write("RPRT 1\n");
     }
 }
@@ -282,6 +406,9 @@ void RemoteControl::setSignalLevel(float level)
 void RemoteControl::setMode(int mode)
 {
     rc_mode = mode;
+
+    if (rc_mode == 0)
+        audio_recorder_status = false;
 }
 
 /*! \brief New remote frequency received. */
@@ -299,12 +426,40 @@ void RemoteControl::setNewRemoteFreq(qint64 freq)
     {
         // move rx freqeucy and let MainWindow deal with it
         // (will usually change hardware PLL)
+        // reset the filter_offset, otherwise the MainWindo will preserve it
+        rc_filter_offset = 0;
+        emit newFilterOffset(rc_filter_offset);
         emit newFrequency(freq);
     }
 
     rc_freq = freq;
 }
 
+/*! \brief Set squelch level (from mainwindow). */
+void RemoteControl::setSquelchLevel(double level)
+{
+    squelch_level = level;
+}
+
+/*! \brief Start audio recorder (from mainwindow). */
+void RemoteControl::startAudioRecorder(QString unused)
+{
+    if (rc_mode > 0)
+        audio_recorder_status = true;
+}
+
+/*! \brief Stop audio recorder (from mainwindow). */
+void RemoteControl::stopAudioRecorder()
+{
+    audio_recorder_status = false;
+}
+
+/*! \brief Set receiver status (from mainwindow). */
+void RemoteControl::setReceiverStatus(bool enabled)
+{
+    receiver_running = enabled;
+}
+
 
 /*! \brief Convert mode string to enum (DockRxOpt::rxopt_mode_idx)
  *  \param mode The Hamlib rigctld compatible mode string
@@ -313,17 +468,10 @@ void RemoteControl::setNewRemoteFreq(qint64 freq)
  * Following mode strings are recognized: OFF, RAW, AM, FM, WFM,
  * WFM_ST, WFM_ST_OIRT, LSB, USB, CW, CWL, CWU.
  */
-int RemoteControl::modeStrToInt(const char *buffer)
+int RemoteControl::modeStrToInt(QString mode_str)
 {
-    QStringList    str_list;
-    QString        mode_str;
-    int            mode_int = 0;
+    int mode_int = -1;
 
-    str_list = QString(buffer).split(' ', QString::SkipEmptyParts);
-    if (str_list.size() < 2)
-        return 0;
-
-    mode_str = str_list.at(1).trimmed();
     if (mode_str.compare("OFF", Qt::CaseInsensitive) == 0)
     {
         mode_int = 0;
