@@ -36,6 +36,8 @@
 
 #ifdef WITH_PULSEAUDIO
 #include "pulseaudio/pa_device_list.h"
+#elif WITH_PORTAUDIO
+#include "portaudio/device_list.h"
 #elif defined(GQRX_OS_MACX)
 #include "osxaudio/device_list.h"
 #endif
@@ -44,69 +46,28 @@
 #include "ui_ioconfig.h"
 
 
-CIoConfig::CIoConfig(QSettings *settings, QWidget *parent) :
+CIoConfig::CIoConfig(QSettings * settings,
+                     std::map<QString, QVariant> &devList,
+                     QWidget *parent) :
     QDialog(parent),
     ui(new Ui::CIoConfig),
     m_settings(settings)
 {
     unsigned int i=0;
     QString devstr;
-    QString devlabel;
     bool cfgmatch=false; //flag to indicate that device from config was found
 
     ui->setupUi(this);
 
     QString indev = settings->value("input/device", "").toString();
 
-    // automatic discovery of FCD does not work on Mac
-    // so we do it ourselves
-#if defined(GQRX_OS_MACX)
-    osxaudio_device_list devices;
-    inDevList = devices.get_input_devices();
-
-    string this_dev;
-    for (i = 0; i < inDevList.size(); i++)
+    // insert the device list in device combo box
+    std::map<QString, QVariant>::iterator I = devList.begin();
+    i = 0;
+    while (I != devList.end())
     {
-        this_dev = inDevList[i].get_name();
-        if (this_dev.find("FUNcube Dongle V1.0") != string::npos)
-        {
-            devstr = "fcd,type=1,device='FUNcube Dongle V1.0'";
-            ui->inDevCombo->addItem("FUNcube Dongle V1.0", QVariant(devstr));
-        }
-        else if (this_dev.find("FUNcube Dongle V2.0") != string::npos)
-        {
-            devstr = "fcd,type=2,device='FUNcube Dongle V2.0'";
-            ui->inDevCombo->addItem("FUNcube Dongle V2.0", QVariant(devstr));
-        }
-
-        if (indev == QString(inDevList[i].get_name().c_str()))
-        {
-            ui->inDevCombo->setCurrentIndex(i);
-            ui->inDevEdit->setText(devstr);
-            cfgmatch = true;
-        }
-    }
-#endif
-
-    // Get list of input devices discovered by gr-osmosdr and store them in
-    // the input device selector together with the device descriptor strings
-    osmosdr::devices_t devs = osmosdr::device::find();
-
-    qDebug() << __FUNCTION__ << ": Available input devices:";
-    BOOST_FOREACH(osmosdr::device_t &dev, devs)
-    {
-        if (dev.count("label"))
-        {
-            devlabel = QString(dev["label"].c_str());
-            dev.erase("label");
-        }
-        else
-        {
-            devlabel = "Unknown";
-        }
-
-        devstr = QString(dev.to_string().c_str());
-        ui->inDevCombo->addItem(devlabel, QVariant(devstr));
+        devstr = (*I).second.toString();
+        ui->inDevCombo->addItem((*I).first, devstr);
 
         // is this the device stored in config?
         if (indev == devstr)
@@ -115,20 +76,11 @@ CIoConfig::CIoConfig(QSettings *settings, QWidget *parent) :
             ui->inDevEdit->setText(devstr);
             cfgmatch = true;
         }
-
-        qDebug() << "   " << i << ":"  << devlabel;
+        ++I;
         ++i;
-
-        // Following code could be used for multiple matches
-        /* QStringList list;
-        int pos = 0;
-        while ((pos = rx.indexIn(devstr, pos)) != -1) {
-            list << rx.cap(1);
-            pos += rx.matchedLength();
-        } */
-
     }
 
+
     ui->inDevCombo->addItem(tr("Other..."), QVariant(""));
 
     // If device string from config is not one of the detected devices
@@ -164,13 +116,13 @@ CIoConfig::CIoConfig(QSettings *settings, QWidget *parent) :
     ui->bwSpinBox->setValue(1.0e-6*settings->value("input/bandwidth", 0.0).toDouble());
 
     // LNB LO
-    ui->loSpinBox->setValue(1.0e-6*settings->value("input/lnb_lo", 0.0).toDouble());
+    ui->loSpinBox->setValue(1.0e-6 * settings->value("input/lnb_lo", 0.0).toDouble());
 
     // Output device
     QString outdev = settings->value("output/device", "").toString();
 
+     // get list of audio output devices
 #ifdef WITH_PULSEAUDIO
-    // get list of output devices
     pa_device_list devices;
     outDevList = devices.get_output_devices();
 
@@ -186,10 +138,23 @@ CIoConfig::CIoConfig(QSettings *settings, QWidget *parent) :
         if (outdev == QString(outDevList[i].get_name().c_str()))
             ui->outDevCombo->setCurrentIndex(i+1);
     }
+#elif WITH_PORTAUDIO
+    portaudio_device_list   devices;
+
+    outDevList = devices.get_output_devices();
+    for (i = 0; i < outDevList.size(); i++)
+    {
+        ui->outDevCombo->addItem(QString(outDevList[i].get_description().c_str()));
+
+        // note that item #i in devlist will be item #(i+1)
+        // in combo box due to "default"
+        if (outdev == QString(outDevList[i].get_name().c_str()))
+            ui->outDevCombo->setCurrentIndex(i+1);
+    }
+    //ui->outDevCombo->setEditable(true);
 
 #elif defined(GQRX_OS_MACX)
-    // get list of output devices
-    // (already defined) osxaudio_device_list devices;
+    osxaudio_device_list devices;
     outDevList = devices.get_output_devices();
 
     qDebug() << __FUNCTION__ << ": Available output devices:";
@@ -205,6 +170,8 @@ CIoConfig::CIoConfig(QSettings *settings, QWidget *parent) :
     }
 
 #else
+    ui->outDevCombo->addItem(settings->value("output/device", "Default").toString(),
+                             settings->value("output/device", "Default").toString());
     ui->outDevCombo->setEditable(true);
 #endif // WITH_PULSEAUDIO
 
@@ -221,24 +188,97 @@ CIoConfig::~CIoConfig()
     delete ui;
 }
 
+/**
+ * @brief get the list of devices
+ */
+void CIoConfig::getDeviceList(std::map<QString, QVariant> &devList)
+{
+    unsigned int i=0;
+    QString devstr;
+    QString devlabel;
 
-/** @brief Save configuration. */
+#if defined(GQRX_OS_MACX)
+    // automatic discovery of FCD does not work on Mac
+    // so we do it ourselves
+    osxaudio_device_list devices;
+    vector<osxaudio_device> inDevList = devices.get_input_devices();
+
+    string this_dev;
+    for (i = 0; i < inDevList.size(); i++)
+    {
+        this_dev = inDevList[i].get_name();
+        if (this_dev.find("FUNcube Dongle V1.0") != string::npos)
+        {
+            devstr = "fcd,type=1,device='FUNcube Dongle V1.0'";
+            devList.insert(std::pair<QString, QVariant>(QString("FUNcube Dongle V1.0"), QVariant(devstr)));
+        }
+        else if (this_dev.find("FUNcube Dongle V1_0") != string::npos)      // since OS X 10.11.4
+        {
+            devstr = "fcd,type=1,device='FUNcube Dongle V1_0'";
+            devList.insert(std::pair<QString, QVariant>(QString("FUNcube Dongle V1_0"), QVariant(devstr)));
+        }
+        else if (this_dev.find("FUNcube Dongle V2.0") != string::npos)
+        {
+            devstr = "fcd,type=2,device='FUNcube Dongle V2.0'";
+            devList.insert(std::pair<QString, QVariant>(QString("FUNcube Dongle V2.0"), QVariant(devstr)));
+        }
+        else if (this_dev.find("FUNcube Dongle V2_0") != string::npos)      // since OS X 10.11.4
+        {
+            devstr = "fcd,type=2,device='FUNcube Dongle V2_0'";
+            devList.insert(std::pair<QString, QVariant>(QString("FUNcube Dongle V2_0"), QVariant(devstr)));
+        }
+    }
+#endif
+
+    // Get list of input devices discovered by gr-osmosdr and store them in
+    // the device list together with the device descriptor strings
+    osmosdr::devices_t devs = osmosdr::device::find();
+
+    qDebug() << __FUNCTION__ << ": Available input devices:";
+    BOOST_FOREACH(osmosdr::device_t &dev, devs)
+    {
+        if (dev.count("label"))
+        {
+            devlabel = QString(dev["label"].c_str());
+            dev.erase("label");
+        }
+        else
+        {
+            devlabel = "Unknown";
+        }
+
+        devstr = QString(dev.to_string().c_str());
+        devList.insert(std::pair<QString, QVariant>(devlabel, devstr));
+
+        qDebug() << "   " << i << ":"  << devlabel;
+        ++i;
+    }
+}
+
+/** Save configuration. */
 void CIoConfig::saveConfig()
 {
     int         idx;
     int         int_val;
+    bool        conv_ok;
 
     qDebug() << __FUNCTION__;
 
     idx = ui->outDevCombo->currentIndex();
 
+#if defined(WITH_PULSEAUDIO) || defined(GQRX_OS_MACX)
     if (idx > 0)
     {
-#if defined(WITH_PULSEAUDIO) || defined(GQRX_OS_MACX)
-        qDebug() << "Output device" << idx << ":" << QString(outDevList[idx-1].get_name().c_str());
-        m_settings->setValue("output/device", QString(outDevList[idx-1].get_name().c_str()));
-#endif
+          qDebug() << "Output device" << idx << ":" << QString(outDevList[idx-1].get_name().c_str());
+          m_settings->setValue("output/device", QString(outDevList[idx-1].get_name().c_str()));
     }
+#else
+    if (idx > 0 || ui->outDevCombo->currentText() != "Default")
+    {
+        qDebug() << "Output device:" << ui->outDevCombo->currentText();
+        m_settings->setValue("output/device", ui->outDevCombo->currentText());
+    }
+#endif
     else
     {
         m_settings->remove("output/device");
@@ -256,10 +296,11 @@ void CIoConfig::saveConfig()
     value = (qint64)(ui->loSpinBox->value()*1.e6);
     if (value)
         m_settings->setValue("input/lnb_lo", value);
+    else
+        m_settings->remove("input/lnb_lo");
 
-    bool ok=false;
-    int_val = ui->inSrCombo->currentText().toInt(&ok);
-    if (ok)
+    int_val = ui->inSrCombo->currentText().toInt(&conv_ok);
+    if (conv_ok)
         m_settings->setValue("input/sample_rate", int_val);
     else
         m_settings->remove("input/sample_rate");
@@ -282,22 +323,11 @@ void CIoConfig::updateInputSampleRates(int rate)
     ui->inSrCombo->clear();
 
     if (ui->inDevEdit->text().isEmpty())
-    {
         return;
-    }
-
-    /** FIXME: this code crashes on RTL device so we use fixed rates **/
-    //osmosdr_source_c_sptr src = osmosdr_make_source_c(ui->inDevEdit->text().toStdString());
-    //osmosdr::meta_range_t rates = src->get_sample_rates();
-    //BOOST_FOREACH(osmosdr::range_t &rate, rates)
-    //{
-    //    ui->inSrCombo->addItem(QString("%1 kHz").arg(rate.start()/1000, 0, 'f', 0));
-    //}
-    //src.reset();
 
     if (ui->inDevEdit->text().contains("fcd"))
     {
-        if (ui->inDevCombo->currentText().contains("V2.0"))
+        if (ui->inDevCombo->currentText().contains("V2"))   // V2.0 or V2_0
         {
             ui->inSrCombo->addItem("192000");
         }
@@ -371,9 +401,6 @@ void CIoConfig::updateInputSampleRates(int rate)
     }
     else if (ui->inDevEdit->text().contains("sdr-iq"))
     {
-        if (rate > 0)
-            ui->inSrCombo->addItem(QString("%1").arg(rate));
-
         ui->inSrCombo->addItem("8138");
         ui->inSrCombo->addItem("16276");
         ui->inSrCombo->addItem("37793");
@@ -381,12 +408,16 @@ void CIoConfig::updateInputSampleRates(int rate)
         ui->inSrCombo->addItem("111111");
         ui->inSrCombo->addItem("158730");
         ui->inSrCombo->addItem("196078");
+        if (rate > 0)
+        {
+            ui->inSrCombo->insertItem(0, QString("%1").arg(rate));
+            ui->inSrCombo->setCurrentIndex(0);
+        }
+        else
+            ui->inSrCombo->setCurrentIndex(4); // select 111.111 kHz
     }
     else if (ui->inDevEdit->text().contains("sdr-ip"))
     {
-        if (rate > 0)
-            ui->inSrCombo->addItem(QString("%1").arg(rate));
-
         ui->inSrCombo->addItem("31250");
         ui->inSrCombo->addItem("32000");
         ui->inSrCombo->addItem("40000");
@@ -406,12 +437,16 @@ void CIoConfig::updateInputSampleRates(int rate)
         ui->inSrCombo->addItem("1000000");
         ui->inSrCombo->addItem("1600000");
         ui->inSrCombo->addItem("2000000");
+        if (rate > 0)
+        {
+            ui->inSrCombo->insertItem(0, QString("%1").arg(rate));
+            ui->inSrCombo->setCurrentIndex(0);
+        }
+        else
+            ui->inSrCombo->setCurrentIndex(11); // select 250 kHz
     }
     else if (ui->inDevEdit->text().contains("netsdr"))
     {
-        if (rate > 0)
-            ui->inSrCombo->addItem(QString("%1").arg(rate));
-
         ui->inSrCombo->addItem("32000");
         ui->inSrCombo->addItem("40000");
         ui->inSrCombo->addItem("50000");
@@ -432,6 +467,13 @@ void CIoConfig::updateInputSampleRates(int rate)
         ui->inSrCombo->addItem("1000000");
         ui->inSrCombo->addItem("1250000");
         ui->inSrCombo->addItem("2000000");
+        if (rate > 0)
+        {
+            ui->inSrCombo->insertItem(0, QString("%1").arg(rate));
+            ui->inSrCombo->setCurrentIndex(0);
+        }
+        else
+            ui->inSrCombo->setCurrentIndex(11); // select 250 kHz
     }
     else if (ui->inDevEdit->text().contains("cloudiq"))
     {
@@ -449,9 +491,12 @@ void CIoConfig::updateInputSampleRates(int rate)
         ui->inSrCombo->addItem("1228800");
         ui->inSrCombo->addItem("1807058");
         if (rate > 0)
+        {
             ui->inSrCombo->insertItem(0, QString("%1").arg(rate));
+            ui->inSrCombo->setCurrentIndex(0);
+        }
         else
-            ui->inSrCombo->setCurrentIndex(4); // select 370 kHz
+            ui->inSrCombo->setCurrentIndex(4); // select 240 kHz
     }
     else if (ui->inDevEdit->text().contains("airspy"))
     {
@@ -459,6 +504,8 @@ void CIoConfig::updateInputSampleRates(int rate)
             ui->inSrCombo->addItem(QString("%1").arg(rate));
 
         ui->inSrCombo->addItem("2500000");
+        ui->inSrCombo->addItem("3000000");
+        ui->inSrCombo->addItem("6000000");
         ui->inSrCombo->addItem("10000000");
     }
     else if (ui->inDevEdit->text().contains("redpitaya"))
@@ -470,10 +517,46 @@ void CIoConfig::updateInputSampleRates(int rate)
         ui->inSrCombo->addItem("500000");
         ui->inSrCombo->addItem("1250000");
         if (rate > 0)
-            ui->inSrCombo->addItem(QString("%1").arg(rate));
+        {
+            ui->inSrCombo->insertItem(0, QString("%1").arg(rate));
+            ui->inSrCombo->setCurrentIndex(0);
+        }
         else
             ui->inSrCombo->setCurrentIndex(3); // select 250 kHz
     }
+    else if (ui->inDevEdit->text().contains("sdrplay"))
+    {
+        ui->inSrCombo->addItem("222222");
+        ui->inSrCombo->addItem("333333");
+        ui->inSrCombo->addItem("428571");
+        ui->inSrCombo->addItem("500000");
+        ui->inSrCombo->addItem("571429");
+        ui->inSrCombo->addItem("750000");
+        ui->inSrCombo->addItem("875000");
+        ui->inSrCombo->addItem("1000000");
+        ui->inSrCombo->addItem("1536000");
+        ui->inSrCombo->addItem("2048000");
+        ui->inSrCombo->addItem("5000000");
+        ui->inSrCombo->addItem("6000000");
+        ui->inSrCombo->addItem("7000000");
+        ui->inSrCombo->addItem("8000000");
+        ui->inSrCombo->addItem("9000000");
+        ui->inSrCombo->addItem("10000000");
+        ui->inSrCombo->addItem("11000000");
+        ui->inSrCombo->addItem("12000000");
+        if (rate > 0)
+        {
+            ui->inSrCombo->insertItem(0, QString("%1").arg(rate));
+            ui->inSrCombo->setCurrentIndex(0);
+        }
+        else
+            ui->inSrCombo->setCurrentIndex(9); // select 2048 kHz
+    }
+    else
+    {
+        if (rate > 0)
+            ui->inSrCombo->addItem(QString("%1").arg(rate));
+    }
 
     updateDecimations();
 }
@@ -511,10 +594,10 @@ void CIoConfig::updateDecimations(void)
         ui->decimCombo->addItem("64", 0);
     if (rate >= 6144000)
         ui->decimCombo->addItem("128", 0);
-    if (rate >= 12288000)
-        ui->decimCombo->addItem("256", 0);
-    if (rate >= 24576000)
-        ui->decimCombo->addItem("512", 0);
+//    if (rate >= 12288000)
+//        ui->decimCombo->addItem("256", 0);
+//    if (rate >= 24576000)
+//        ui->decimCombo->addItem("512", 0);
 
     decimationChanged(0);
 }
@@ -588,9 +671,7 @@ void CIoConfig::decimationChanged(int index)
                                    arg(quad_rate * 1.e-3, 0, 'f', 3));
 }
 
-/**
- * @brief Convert a combo box index to decimation.
- */
+/** Convert a combo box index to decimation. */
 int CIoConfig::idx2decim(int idx) const
 {
     if (idx < 1)
@@ -599,9 +680,7 @@ int CIoConfig::idx2decim(int idx) const
     return (1 << idx);
 }
 
-/**
- * @brief Convert a decimation to a combobox index
- */
+/** Convert a decimation to a combobox index */
 int CIoConfig::decim2idx(int decim) const
 {
     int         idx;
